package edu.nwu.ccl.nlogo.extensions.texas ;

public strictfp class TestIOUtils
	extends junit.framework.TestCase
{
	public TestIOUtils( String name )
	{
		super( name ) ;
	}

	/// helpers
	private static final String[][] simpleLines = {
		{ "hi there." , "how's it going" , "5" , "6" } ,
		{ "line2" , "stuff" , "more stuff" , "45" , "6.8" } ,
		{ "line3" , "other stuff" , "more stuff" , "hi" , "/twe" } } ;
	
	private static final String[][] complexLines = {
			{ "hi there." , "how's it going" , "5" , "6" } ,
			{ "line2" , "stuff" , "more stuff" , "45" , "6.8" } ,
			{ "" } ,
			{ "line4" , "other stuff" , "more stuff" , "hi" , "/twe" } } ;

	private ListStringProducer fileCreator( String[][] lines , String lineDelim )
	{
		StringBuffer buf = new StringBuffer() ;
		org.nlogo.api.LogoList linesList = new org.nlogo.api.LogoList() ;
		for( int i = 0 ; i < lines.length ; i++ )
		{
			ListStringProducer temp = lineCreator( lines[ i ] ) ;
			linesList.add( temp.expectedList ) ;
			buf.append( temp.inputString ) ;
			if( i + 1 < lines.length )
			{
				buf.append( lineDelim ) ;
			}
		}
		return new ListStringProducer( linesList , buf ) ;
	}
	
	private ListStringProducer lineCreator( String[] tabs )
	{
		StringBuffer buf = new StringBuffer() ;
		org.nlogo.api.LogoList l = new org.nlogo.api.LogoList( tabs.length ) ;
		for( int i = 0 ; i < tabs.length ; i++ )
		{
			buf.append( tabs[ i ] ) ;
			l.add( tabs[ i ] ) ;
			if( i + 1 < tabs.length )
			{
				buf.append( "\t" ) ;
			}
		}
		return new ListStringProducer( l , buf ) ;
	}

	private void splitIntoStringsHelper( String s , 
		org.nlogo.api.LogoList expected )
	{
		assertEquals( expected, IOUtils.splitIntoLists( 
			s , "\t" ) ) ;
	}

	private class ListStringProducer
	{
		public org.nlogo.api.LogoList expectedList ;
		public String inputString ;
		
		public ListStringProducer( org.nlogo.api.LogoList l , 
			StringBuffer buf )
		{
			expectedList = l ;
			inputString = buf.toString() ;
		}
	}

	/// tests
	public void testSplitEmptyString()
	{
		org.nlogo.api.LogoList expected = new org.nlogo.api.LogoList() ;
		org.nlogo.api.LogoList l = new org.nlogo.api.LogoList() ;
		l.add( "" ) ;
		expected.add( l ) ;
		splitIntoStringsHelper( "" , expected ) ;
	}
	
	public void testSplitEmptyLine()
	{
		String[] tabs = { "" } ;
		org.nlogo.api.LogoList expected = new org.nlogo.api.LogoList() ;
		ListStringProducer temp = lineCreator( tabs ) ;
		expected.add( temp.expectedList ) ;
		splitIntoStringsHelper( temp.inputString , expected ) ;
	}

	public void testSplitLine()
	{
		String[] tabs = { "hi there." , "how's it going" , "5", "6" } ;
		org.nlogo.api.LogoList expected = new org.nlogo.api.LogoList() ;
		ListStringProducer temp = lineCreator( tabs ) ;
		expected.add( temp.expectedList ) ;
		splitIntoStringsHelper( temp.inputString , expected ) ;
	}

	public void testSplitFile1()
	{
		ListStringProducer file = fileCreator( simpleLines , "\n" ) ;
		splitIntoStringsHelper( file.inputString , file.expectedList ) ;
	}

	public void testSplitFile2()
	{
		ListStringProducer file = fileCreator( complexLines , "\n" ) ;
		splitIntoStringsHelper( file.inputString , file.expectedList ) ;
	}

	public void testSplitFile3()
	{
		ListStringProducer file = fileCreator( complexLines , "\r" ) ;
		splitIntoStringsHelper( file.inputString , file.expectedList ) ;
	}
	
	public void testSplitFile4()
	{
		ListStringProducer file = fileCreator( complexLines , "\r\n" ) ;
		splitIntoStringsHelper( file.inputString , file.expectedList ) ;
	}

	/// more tests

	public void testReadURL1() throws java.io.IOException
	{
		try
		{
			IOUtils.readURL( "ghjkfghjfh" ) ;
			assertTrue( false ) ;
		}
		catch( java.net.MalformedURLException ex )
		{
			org.nlogo.util.Exceptions.ignore( ex ) ;
		}
	}
	
	public void testReadURL2()
	{
		try
		{
			IOUtils.readURL( "http://www.ghjfgjkhfgjkfjkgfjkgfhjkgkfjghjkfghjkghjkajkahjkkjjkfhdg5789454.com/" ) ;
			assertTrue( false ) ;
		}
		catch( java.io.IOException ex )
		{
			org.nlogo.util.Exceptions.ignore( ex ) ;
		}
	}
	
}
